###############   PSQF 6249 FA & SEM Example 7a CFA Multiple-Group Invariance using R   #############

# Set width of output and  number of digits to be printed
options("width"=140, "digits"=6) 

# Use this to turn off scientific notation: 
# "fixed notation will be preferred unless it is more than 'scipen' digits wide"
options(scipen=9)

# turn off stars for different alpha levels
options(show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("ggplot2")) install.packages("ggplot2")
library(ggplot2) # To make pictures

if (!require("lavaan")) install.packages("lavaan")
library(lavaan) # To fit SEMs

# Other packages not used here, but which are pry useful

#if (!require("semTools")) install.packages("semTools")
#library(semTools) # lavaan add-ons (multiple imputation, moderation)

#if (!require("MplusAutomation")) install.packages("MplusAutomation")
#library(lavaan) # To use Mplus within R

#########################################################################################################

# Define variables for working directory and data name
filesave = "C:\\Dropbox/22_PSQF6249/Depression/PSQF6249_Example7a/"
filename = "Example7a.xlsx"
setwd(dir=filesave)

# Import depression item-level data
Example7a = read_excel(paste0(filesave,filename)) 
# Convert to data frame as data frame to use for analysis
Example7a = as.data.frame(Example7a)
# Recode sex to specify each group name (to show in output)
Example7a$sex[which(Example7a$sex==0)] = "0Women"
Example7a$sex[which(Example7a$sex==1)] = "1Men"  

# Open external file to save results to
txtStart(file=paste0(filesave,"R_Example7a_CFA_MultipleGroup_Output.txt"))

#### General notes about lavaan syntax objects:
# Once factors are defined they can be used in syntax as if they were observed variables
# Multiple commands can go on a single line separated by ; 
# Parameters can be fixed to constant values by using the * command
# Start values can be specified by using value* in front of parameter 
# Labels can be implemented with (label)* in front of parameter

# Example of how to concatenate syntax strings into one model
#Model <- sem(model=c(syntax1, syntax2), data = myData)

#########################################################################################################
# Measurement Invariance Model 1: Configural Invariance Model
# all parameters are estimated separately across groups except those for identification

ConfiguralSyntax = "
#===================================================================================================
# Factor loadings all freely estimated in both groups with label for each group
depress =~ c(L1W, L1M)*item1 + c(L2W, L2M)*item2 + c(L3W, L3M)*item3 + 
           c(L4W, L4M)*item4 + c(L5W, L5M)*item5 + c(L6W, L6M)*item6 + 
           c(L7W, L7M)*item7 + c(L8W, L8M)*item8 + c(L9W, L9M)*item9
#===================================================================================================
# Item intercepts all freely estimated in both groups with label for each group
item1 ~ c(I1W, I1M)*1; item2 ~ c(I2W, I2M)*1; item3 ~ c(I3W, I3M)*1; 
item4 ~ c(I4W, I4M)*1; item5 ~ c(I5W, I5M)*1; item6 ~ c(I6W, I6M)*1; 
item7 ~ c(I7W, I7M)*1; item8 ~ c(I8W, I8M)*1; item9 ~ c(I9W, I9M)*1;
#===================================================================================================
# Residual variances all freely estimated with label for each group
item1 ~~ c(E1W, E1M)*item1; item2 ~~ c(E2W, E2M)*item2; item3 ~~ c(E3W, E3M)*item3; 
item4 ~~ c(E4W, E4M)*item4; item5 ~~ c(E5W, E5M)*item5; item6 ~~ c(E6W, E6M)*item6; 
item7 ~~ c(E7W, E7M)*item7; item8 ~~ c(E8W, E8M)*item8; item9 ~~ c(E9W, E9M)*item9;
#===================================================================================================
# Residual covariance freely estimated in both groups with label for each group
item1 ~~ c(EC12W, EC12M)*item2
#===================================================================================================
# Factor variance fixed to 1 for identification in each group
depress ~~ c(1,1)*depress
#===================================================================================================
# Factor mean fixed to zero for identification in each group
depress ~ c(0,0)*0
#===================================================================================================
"
Configural = lavaan(model=ConfiguralSyntax, data=Example7a, estimator="MLR", mimic="mplus", group="sex")
summary(Configural, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)

txtComment(" ") # insert blank space in output

#########################################################################################################
# Measurement Invariance Model 2: Metric Invariance Model
# constrain factor loadings equal across groups, estimate factor variance in alternative group

MetricSyntax = "
#===================================================================================================
# Factor loadings NOW CONSTRAINED EQUAL ACROSS GROUPS via same label
depress =~ c(L1, L1)*item1 + c(L2, L2)*item2 + c(L3, L3)*item3 + 
           c(L4, L4)*item4 + c(L5, L5)*item5 + c(L6, L6)*item6 + 
           c(L7, L7)*item7 + c(L8, L8)*item8 + c(L9, L9)*item9
#===================================================================================================
# Item intercepts all freely estimated in both groups with label for each group
item1 ~ c(I1W, I1M)*1; item2 ~ c(I2W, I2M)*1; item3 ~ c(I3W, I3M)*1; 
item4 ~ c(I4W, I4M)*1; item5 ~ c(I5W, I5M)*1; item6 ~ c(I6W, I6M)*1; 
item7 ~ c(I7W, I7M)*1; item8 ~ c(I8W, I8M)*1; item9 ~ c(I9W, I9M)*1;
#===================================================================================================
# Residual variances all freely estimated with label for each group
item1 ~~ c(E1W, E1M)*item1; item2 ~~ c(E2W, E2M)*item2; item3 ~~ c(E3W, E3M)*item3; 
item4 ~~ c(E4W, E4M)*item4; item5 ~~ c(E5W, E5M)*item5; item6 ~~ c(E6W, E6M)*item6; 
item7 ~~ c(E7W, E7M)*item7; item8 ~~ c(E8W, E8M)*item8; item9 ~~ c(E9W, E9M)*item9;
#===================================================================================================
# Residual covariance freely estimated in both groups with label for each group
item1 ~~ c(EC12W, EC12M)*item2
#===================================================================================================
# Factor variance still fixed to 1 for identification in women but NOW ESTIMATED in men 
depress ~~ c(1,NA)*depress
#===================================================================================================
# Factor mean still fixed to zero for identification in each group
depress ~ c(0,0)*0
#===================================================================================================
"
Metric = lavaan(model=MetricSyntax, data=Example7a, estimator="MLR", mimic="mplus", group="sex")
summary(Metric, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual loadings
modificationindices(object=Metric, op="=~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is Metric worse than Configural?")
anova(Configural, Metric)

txtComment(" ") # insert blank space in output

#########################################################################################################
# Measurement Invariance Model 3: Scalar Invariance Model (Version A)
# constrain intercepts equal across groups, estimate factor mean in alternative group

ScalarASyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label
depress =~ c(L1, L1)*item1 + c(L2, L2)*item2 + c(L3, L3)*item3 + 
           c(L4, L4)*item4 + c(L5, L5)*item5 + c(L6, L6)*item6 + 
           c(L7, L7)*item7 + c(L8, L8)*item8 + c(L9, L9)*item9
#===================================================================================================
# Item intercepts NOW CONSTRAINED EQUAL ACROSS GROUPS via same label
item1 ~ c(I1, I1)*1; item2 ~ c(I2, I2)*1; item3 ~ c(I3, I3)*1; 
item4 ~ c(I4, I4)*1; item5 ~ c(I5, I5)*1; item6 ~ c(I6, I6)*1; 
item7 ~ c(I7, I7)*1; item8 ~ c(I8, I8)*1; item9 ~ c(I9, I9)*1;
#===================================================================================================
# Residual variances all freely estimated with label for each group
item1 ~~ c(E1W, E1M)*item1; item2 ~~ c(E2W, E2M)*item2; item3 ~~ c(E3W, E3M)*item3; 
item4 ~~ c(E4W, E4M)*item4; item5 ~~ c(E5W, E5M)*item5; item6 ~~ c(E6W, E6M)*item6; 
item7 ~~ c(E7W, E7M)*item7; item8 ~~ c(E8W, E8M)*item8; item9 ~~ c(E9W, E9M)*item9;
#===================================================================================================
# Residual covariance freely estimated in both groups with label for each group
item1 ~~ c(EC12W, EC12M)*item2
#===================================================================================================
# Factor variance still fixed to 1 for identification in women but still estimated in men
depress ~~ c(1,NA)*depress
#===================================================================================================
# Factor mean still fixed to zero for identification in women but NOW ESTIMATED in men
depress ~ c(0,NA)*1
#===================================================================================================
"
ScalarA = lavaan(model=ScalarASyntax, data=Example7a, estimator="MLR", mimic="mplus", group="sex")
summary(ScalarA, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual intercepts
modificationindices(object=ScalarA, op="~1", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarA worse than Metric?")
anova(Metric, ScalarA)

txtComment(" ") # insert blank space in output

#########################################################################################################
# Measurement Invariance Model 3: Scalar Invariance Model (Version B)
# constrain intercepts equal across groups except item 7, estimate factor mean in alternative group


ScalarBSyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label
depress =~ c(L1, L1)*item1 + c(L2, L2)*item2 + c(L3, L3)*item3 + 
           c(L4, L4)*item4 + c(L5, L5)*item5 + c(L6, L6)*item6 + 
           c(L7, L7)*item7 + c(L8, L8)*item8 + c(L9, L9)*item9
#===================================================================================================
# Item intercepts NOW CONSTRAINED EQUAL ACROSS GROUPS via same label EXCEPT ITEM 7
item1 ~ c(I1, I1)*1;   item2 ~ c(I2, I2)*1; item3 ~ c(I3, I3)*1; 
item4 ~ c(I4, I4)*1;   item5 ~ c(I5, I5)*1; item6 ~ c(I6, I6)*1; 
item7 ~ c(I7W, I7M)*1; item8 ~ c(I8, I8)*1; item9 ~ c(I9, I9)*1;
#===================================================================================================
# Residual variances all freely estimated with label for each group
item1 ~~ c(E1W, E1M)*item1; item2 ~~ c(E2W, E2M)*item2; item3 ~~ c(E3W, E3M)*item3; 
item4 ~~ c(E4W, E4M)*item4; item5 ~~ c(E5W, E5M)*item5; item6 ~~ c(E6W, E6M)*item6; 
item7 ~~ c(E7W, E7M)*item7; item8 ~~ c(E8W, E8M)*item8; item9 ~~ c(E9W, E9M)*item9;
#===================================================================================================
# Residual covariance freely estimated in both groups with label for each group
item1 ~~ c(EC12W, EC12M)*item2
#===================================================================================================
# Factor variance fixed to 1 for identification in women but still estimated in men
depress ~~ c(1,NA)*depress
#===================================================================================================
# Factor mean still fixed to zero for identification in women but NOW ESTIMATED in men
depress ~ c(0,NA)*1
#===================================================================================================
"
ScalarB = lavaan(model=ScalarBSyntax, data=Example7a, estimator="MLR", mimic="mplus", group="sex")
summary(ScalarB, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual parameters
modificationindices(object=ScalarB, op="~1", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ScalarB better than ScalarA?")
anova(ScalarB, ScalarA)

print("LRT: Is ScalarB worse than Metric?")
anova(Metric, ScalarB)

txtComment(" ") # insert blank space in output

#########################################################################################################
# Measurement Invariance Model 4: Residual Variance Invariance Model
# constrain residual variances equal across groups except item 7 (for which scalar did not hold)

ResVarSyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label
depress =~ c(L1, L1)*item1 + c(L2, L2)*item2 + c(L3, L3)*item3 + 
           c(L4, L4)*item4 + c(L5, L5)*item5 + c(L6, L6)*item6 + 
           c(L7, L7)*item7 + c(L8, L8)*item8 + c(L9, L9)*item9
#===================================================================================================
# Item intercepts still constrained equal across groups via same label except item 7
item1 ~ c(I1, I1)*1;   item2 ~ c(I2, I2)*1; item3 ~ c(I3, I3)*1; 
item4 ~ c(I4, I4)*1;   item5 ~ c(I5, I5)*1; item6 ~ c(I6, I6)*1; 
item7 ~ c(I7W, I7M)*1; item8 ~ c(I8, I8)*1; item9 ~ c(I9, I9)*1;
#===================================================================================================
# Residual variances NOW CONSTRAINED EQUAL ACROSS GROUPS via same label EXCEPT ITEM 7
item1 ~~ c(E1, E1)*item1;   item2 ~~ c(E2, E2)*item2; item3 ~~ c(E3, E3)*item3; 
item4 ~~ c(E4, E4)*item4;   item5 ~~ c(E5, E5)*item5; item6 ~~ c(E6, E6)*item6; 
item7 ~~ c(E7W, E7M)*item7; item8 ~~ c(E8, E8)*item8; item9 ~~ c(E9, E9)*item9;
#===================================================================================================
# Residual covariance freely estimated in both groups with label for each group
item1 ~~ c(EC12W, EC12M)*item2
#===================================================================================================
# Factor variance still fixed to 1 for identification in women but still estimated in men
depress ~~ c(1,NA)*depress
#===================================================================================================
# Factor mean still fixed to zero for identification in women but still estimated in men
depress ~ c(0,NA)*1
#===================================================================================================
"
ResVar = lavaan(model=ResVarSyntax, data=Example7a, estimator="MLR", mimic="mplus", group="sex")
summary(ResVar, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual parameters
modificationindices(object=ResVar, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ResVar worse than ScalarB?")
anova(ScalarB, ResVar)

txtComment(" ") # insert blank space in output

#########################################################################################################
# Measurement Invariance Model 5: Residual Covariance Invariance Model
# constrain residual covariance equal across groups 

ResCoVarSyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label
depress =~ c(L1, L1)*item1 + c(L2, L2)*item2 + c(L3, L3)*item3 + 
           c(L4, L4)*item4 + c(L5, L5)*item5 + c(L6, L6)*item6 + 
           c(L7, L7)*item7 + c(L8, L8)*item8 + c(L9, L9)*item9
#===================================================================================================
# Item intercepts still constrained equal across groups via same label except item 7
item1 ~ c(I1, I1)*1;   item2 ~ c(I2, I2)*1; item3 ~ c(I3, I3)*1; 
item4 ~ c(I4, I4)*1;   item5 ~ c(I5, I5)*1; item6 ~ c(I6, I6)*1; 
item7 ~ c(I7W, I7M)*1; item8 ~ c(I8, I8)*1; item9 ~ c(I9, I9)*1;
#===================================================================================================
# Residual variances still constrained equal across groups via same label except item 7
item1 ~~ c(E1, E1)*item1;   item2 ~~ c(E2, E2)*item2; item3 ~~ c(E3, E3)*item3; 
item4 ~~ c(E4, E4)*item4;   item5 ~~ c(E5, E5)*item5; item6 ~~ c(E6, E6)*item6; 
item7 ~~ c(E7W, E7M)*item7; item8 ~~ c(E8, E8)*item8; item9 ~~ c(E9, E9)*item9;
#===================================================================================================
# Residual covariance NOW CONSTRAINED EQUAL ACROSS GROUPS via same label for each group
item1 ~~ c(EC12, EC12)*item2
#===================================================================================================
# Factor variance still fixed to 1 for identification in women but still estimated in men
depress ~~ c(1,NA)*depress
#===================================================================================================
# Factor mean still fixed to zero for identification in women but still estimated in men
depress ~ c(0,NA)*1
#===================================================================================================
"
ResCoVar = lavaan(model=ResCoVarSyntax, data=Example7a, estimator="MLR", mimic="mplus", group="sex")
summary(ResCoVar, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual parameters
modificationindices(object=ResCoVar, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is ResCoVar worse than ResVar?")
anova(ResVar, ResCoVar)

txtComment(" ") # insert blank space in output


#########################################################################################################
# Structural Invariance Model 6: Factor Variance Invariance Model
# constrain factor variance equal across groups 

FactorVarSyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label
depress =~ c(L1, L1)*item1 + c(L2, L2)*item2 + c(L3, L3)*item3 + 
           c(L4, L4)*item4 + c(L5, L5)*item5 + c(L6, L6)*item6 + 
           c(L7, L7)*item7 + c(L8, L8)*item8 + c(L9, L9)*item9
#===================================================================================================
# Item intercepts still constrained equal across groups via same label except item 7
item1 ~ c(I1, I1)*1;   item2 ~ c(I2, I2)*1; item3 ~ c(I3, I3)*1; 
item4 ~ c(I4, I4)*1;   item5 ~ c(I5, I5)*1; item6 ~ c(I6, I6)*1; 
item7 ~ c(I7W, I7M)*1; item8 ~ c(I8, I8)*1; item9 ~ c(I9, I9)*1;
#===================================================================================================
# Residual variances still constrained equal across groups via same label except item 7
item1 ~~ c(E1, E1)*item1;   item2 ~~ c(E2, E2)*item2; item3 ~~ c(E3, E3)*item3; 
item4 ~~ c(E4, E4)*item4;   item5 ~~ c(E5, E5)*item5; item6 ~~ c(E6, E6)*item6; 
item7 ~~ c(E7W, E7M)*item7; item8 ~~ c(E8, E8)*item8; item9 ~~ c(E9, E9)*item9;
#===================================================================================================
# Residual covariance not constrained equal via different labels for each group
item1 ~~ c(EC12W, EC12M)*item2
#===================================================================================================
# Factor variance still fixed to 1 for identification in women but NOW FIXED TO 1 in men
depress ~~ c(1,1)*depress
#===================================================================================================
# Factor mean still fixed to zero for identification in women but still estimated in men
depress ~ c(0,NA)*1
#===================================================================================================
"
FactorVar = lavaan(model=FactorVarSyntax, data=Example7a, estimator="MLR", mimic="mplus", group="sex")
summary(FactorVar, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual parameters
modificationindices(object=FactorVar, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is FactorVar worse than ResVar?")
anova(ResVar, FactorVar)

txtComment(" ") # insert blank space in output


#########################################################################################################
# Structural Invariance Model 7: Factor Mean Invariance Model
# constrain factor mean equal across groups 

FactorMeanSyntax = "
#===================================================================================================
# Factor loadings still constrained equal across groups via same label
depress =~ c(L1, L1)*item1 + c(L2, L2)*item2 + c(L3, L3)*item3 + 
           c(L4, L4)*item4 + c(L5, L5)*item5 + c(L6, L6)*item6 + 
           c(L7, L7)*item7 + c(L8, L8)*item8 + c(L9, L9)*item9
#===================================================================================================
# Item intercepts still constrained equal across groups via same label except item 7
item1 ~ c(I1, I1)*1;   item2 ~ c(I2, I2)*1; item3 ~ c(I3, I3)*1; 
item4 ~ c(I4, I4)*1;   item5 ~ c(I5, I5)*1; item6 ~ c(I6, I6)*1; 
item7 ~ c(I7W, I7M)*1; item8 ~ c(I8, I8)*1; item9 ~ c(I9, I9)*1;
#===================================================================================================
# Residual variances still constrained equal across groups via same label except item 7
item1 ~~ c(E1, E1)*item1;   item2 ~~ c(E2, E2)*item2; item3 ~~ c(E3, E3)*item3; 
item4 ~~ c(E4, E4)*item4;   item5 ~~ c(E5, E5)*item5; item6 ~~ c(E6, E6)*item6; 
item7 ~~ c(E7W, E7M)*item7; item8 ~~ c(E8, E8)*item8; item9 ~~ c(E9, E9)*item9;
#===================================================================================================
# Residual covariance not constrained equal via different labels for each group
item1 ~~ c(EC12W, EC12M)*item2
#===================================================================================================
# Factor variance still fixed to 1 for identification in women and still fixed to 1 in men
depress ~~ c(1,1)*depress
#===================================================================================================
# Factor mean still fixed to zero for identification in women but NOW FIXED TO 0 in men
depress ~ c(0,0)*1
#===================================================================================================
"
FactorMean = lavaan(model=FactorMeanSyntax, data=Example7a, estimator="MLR", mimic="mplus", group="sex")
summary(FactorMean, fit.measures=TRUE, rsquare=TRUE, standardized=TRUE)
# Ask for mod indices to find potential non-invariance of individual parameters
modificationindices(object=FactorMean, op="~~", minimum.value=1, free.remove=FALSE, sort.=TRUE)

print("LRT: Is FactorMean worse than FactorVar?")
anova(FactorVar, FactorMean)

txtComment(" ") # insert blank space in output


#########################################################################################################

# Close external file of saved results
txtStop()


